package tags;

import java.io.*;

public abstract class ITBase {
	private String key;
	
	abstract void writeToTag(DataOutput dataoutput)
	throws IOException;
	
	abstract void readFromTag(DataInput datainput)
			throws IOException;
	
	public abstract byte getType();
	
	protected ITBase(String keypar) {
		if(keypar == null) {
			key = "";
		}
		else {
			key = keypar;
		}
	}
	public boolean equals(Object obj){
		if(obj == null || (obj instanceof ITBase)) {
			return false;
		}
		ITBase itbase = (ITBase)obj;
		if(getType() != itbase.getType()) {
			return false;
		}
		return key == null || key.equals(itbase.key);
	}
		
	public ITBase setKey(String keypar) {
		if(keypar == null) {
			key = "";
		}
		else key = keypar;
		return this;
	}
	public String getKey() {
		if(key == null) {
			return "";
		}
		else return key;
	}
	public static ITBase readTag(DataInput datainput) throws IOException {
		byte inbyte = datainput.readByte();
		if(inbyte == 0) {
			return new ITEnd();
		}
		else {
			String n = datainput.readUTF();
			ITBase itbase = createTagType(inbyte,n);
			itbase.readFromTag(datainput);
			return itbase;
		}
	}
	
	public static void writeTag(ITBase itbase, DataOutput dataoutput) throws IOException {
		dataoutput.writeByte(itbase.getType());
		if(itbase.getType() == 0) {
			return;
		}
		else {
			dataoutput.writeUTF(itbase.getKey());
			itbase.writeToTag(dataoutput);
			return;
		}
	}

	private static ITBase createTagType(byte inbyte, String n) {
		switch(inbyte) {
		case 0: // END
			return new ITEnd();
		case 1: // HEAD
			return new ITHead(n);
		case 2: // LIST
			return new ITList(n);
		case 3: // BYTE
			return new ITByte(n);
		case 4: // SHORT
			return new ITShort(n);
		case 5: // INT
			return new ITInt(n);
		case 6: // LONG
			return new ITLong(n);
		case 7: // DOUBLE
			return new ITDouble(n);
		case 8: // FLOAT
			return new ITFloat(n);
		case 9: // BYTE ARRAY
			return new ITByteArray(n);
		case 10: // STRING
			return new ITString(n);
		case 11: // BOOLEAN
		}
		return null;
		}
	public static String getTagName(byte inbyte) {
		switch(inbyte) {
		case 0: // END
			return "End_Tag";
		case 1: // HEAD
			return "Head_Tag";
		case 2: // LIST
			return "List_Tag";
		case 3: // BYTE
			return "Byte_Tag";
		case 4: // SHORT
			return "Short_Tag";
		case 5: // INT
			return "Int_Tag";
		case 6: // LONG
			return "Long_Tag";
		case 7: // DOUBLE
			return "Double_Tag";
		case 8: // FLOAT
			return "Float_Tag";
		case 9: // BYTE ARRAY
			return "ByteArray_Tag";
		case 10: // STRING
			return "String_Tag";
		case 11: // BOOLEAN
			return "Boolean_Tag";
		}
		return "Null_Tag";
	}
	
	
		
			
}